<?php

namespace Drupal\rate_us\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\node\Entity\Node;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class RateUsController.
 *
 * @package Drupal\rate_us\Controller
 */
class RateUsController extends ControllerBase {

  /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Constructs a new RateUsController object.
   *
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   */
  public function __construct(FormBuilderInterface $form_builder) {
    $this->formBuilder = $form_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('form_builder')
    );
  }

  /**
   * Displays the rating form for a specific node.
   *
   * @param \Drupal\node\Entity\Node $node
   *   The node entity.
   *
   * @return array
   *   A render array for the rating form.
   */
  public function ratingForm(Node $node) {
    if ($node) {
      $form = $this->formBuilder->getForm('Drupal\rate_us\Form\RateUsForm', $node);

      $average_rating = $this->getAverageRating($node->id());
      $stars = $this->generateStars($average_rating);

      $rendered_form = \Drupal::service('renderer')->render($form);

      return [
        '#markup' => $this->t('<div class="rate-us-form">Current rating: @stars @form', [
          '@stars' => $stars,
          '@form' => $rendered_form,
        ]),
      ];
    }
    else {
      return [
        '#markup' => $this->t('The specified content is not available for rating.'),
      ];
    }
  }

  /**
   * Calculate the average rating for the node.
   *
   * @param int $entity_id
   *   The entity ID of the node.
   *
   * @return int
   *   The average rating.
   */
  protected function getAverageRating($entity_id) {
    $query = \Drupal::database()->select('rate_us_ratings', 'r')
      ->fields('r', ['rating'])
      ->condition('entity_id', $entity_id)
      ->execute();

    $ratings = $query->fetchAll();
    $total = 0;
    $count = count($ratings);

    if ($count > 0) {
      foreach ($ratings as $rating) {
        $total += $rating->rating;
      }
      return round($total / $count);
    }

    return 0;
  }

  /**
   * Generate a string of stars based on the rating.
   *
   * @param int $rating
   *   The rating value.
   *
   * @return string
   *   A string of stars.
   */
  protected function generateStars($rating) {
    $stars = '';
    for ($i = 1; $i <= 5; $i++) {
      $stars .= ($i <= $rating) ? '🌟' : '☆';
    }
    return $stars;
  }

}
