<?php

namespace Drupal\rate_us\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\Entity\Node;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a star rating form.
 */
class RateUsForm extends FormBase {

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * RateUsForm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('config.factory'));
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'rate_us_form';
  }

  /**
   * Build the rating form.
   */
  public function buildForm(array $form, FormStateInterface $form_state, ?Node $node = NULL) {
    $node_id = $node ? $node->id() : NULL;

    if ($node_id) {
      $form['#node_id'] = $node_id;

      // Get the enabled node types from the configuration.
      $config = $this->configFactory->get('rate_us.settings');
      $enabled_node_types = $config->get('enabled_node_types') ?? [];

      if (empty($enabled_node_types) || !in_array($node->getType(), array_keys($enabled_node_types))) {
        return $form;
      }

      // Existing rating logic.
      $existing_rating = \Drupal::database()->select('rate_us_ratings', 'r')
        ->fields('r', ['rating'])
        ->condition('entity_id', $node->id())
        ->condition('uid', \Drupal::currentUser()->id())
        ->execute()
        ->fetchField();

      $default_value = $existing_rating ? $existing_rating : 0;

      $form['rating'] = [
        '#type' => 'radios',
        '#title' => t('Rate this content'),
        '#options' => [
          1 => t('🌟'),
          2 => t('🌟🌟'),
          3 => t('🌟🌟🌟'),
          4 => t('🌟🌟🌟🌟'),
          5 => t('🌟🌟🌟🌟🌟'),
        ],
        '#default_value' => $default_value,
        '#required' => FALSE,
      ];

      $form['submit'] = [
        '#type' => 'submit',
        '#attributes' => [
          'style' => 'display: none;',
        ],
      ];
    }
    else {
      \Drupal::messenger()->addMessage(t('Invalid node.'));
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $rating = $form_state->getValue('rating');
    $node_id = $form['#node_id'];

    // Load the Node entity using the node ID.
    $node = Node::load($node_id);

    if ($node) {
      $existing_rating = \Drupal::database()->select('rate_us_ratings', 'r')
        ->fields('r', ['id'])
        ->condition('entity_id', $node->id())
        ->condition('uid', \Drupal::currentUser()->id())
        ->execute()
        ->fetchField();

      if ($existing_rating) {
        \Drupal::database()->update('rate_us_ratings')
          ->fields(['rating' => $rating])
          ->condition('id', $existing_rating)
          ->execute();
        \Drupal::messenger()->addMessage(t('Your rating has been updated.'));
      }
      else {
        \Drupal::database()->insert('rate_us_ratings')
          ->fields([
            'entity_id' => $node->id(),
            'rating' => $rating,
            'uid' => \Drupal::currentUser()->id(),
          ])
          ->execute();
        \Drupal::messenger()->addMessage(t('Thank you for your rating.'));
      }
    }
    else {
      \Drupal::messenger()->addMessage(t('Invalid node.'));
    }
  }

}
